;;; This file is part of guile-with-graphics, a graphic library for Guile

;;; Copyright (C) 2009  Luca Saiu

;;; This program is free software: you can redistribute it and/or modify
;;; it under the terms of the GNU General Public License as published by
;;; the Free Software Foundation, either version 3 of the License, or
;;; (at your option) any later version.

;;; This program is distributed in the hope that it will be useful,
;;; but WITHOUT ANY WARRANTY; without even the implied warranty of
;;; MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
;;; GNU General Public License for more details.

;;; You should have received a copy of the GNU General Public License
;;; along with this program.  If not, see <http://www.gnu.org/licenses/>.


;;; This Scheme file should be loaded only *after* having called
;;; initialize-graphics.


;;; I pre-define some useful colors. Of course, always remember that color
;;; coordinates are numbers (floats like 0.7, but also also *rationals* 
;;; like 1/3 are accepted in Scheme) between 0 and 1:
(define black   (make-color 0   0   0))
(define white   (make-color 1/2 1/2 1/2))
(define red     (make-color 1/2 0   0))
(define green   (make-color 0   1/2 0))
(define blue    (make-color 0   0   1/2))
(define cyan    (make-color 0   1/2 1/2))
(define magenta (make-color 1/2 0   1/2))
(define yellow  (make-color 1/2 1/2 0))
(define gray    (make-color 1/2 1/2 1/2))

;;; If the parameter is less than 0, then return 0; if it's greater than
;;; 1, return 1. Otherwise simply return the parameter. This is useful for
;;; keeping color coordinates within the range [0.0, 1.0].
(define (saturate coordinate)
  (cond ((< coordinate 0)
         0)
        ((> coordinate 1)
         1)
        (else
         coordinate)))

;;; Return a new color obtained by multiplying each component of the given
;;; color by the given factor, and saturating:
(define (color-* factor color)
  (let ((red   (color->red color))
        (green (color->green color))
        (blue  (color->blue color)))
    (make-color (saturate (* factor red))
                (saturate (* factor green))
                (saturate (* factor blue)))))

;;; Given a color, return a ligher version of it. For example (light green)
;;; makes a lighter green:
(define (light color)
  (color-* 3/2 color))

;;; Given a color, return a darker version of it. For example (dark green)
;;; makes a darker green:
(define (dark color)
  (color-* 2/3 color))

;; Return a new random color:
(define (random-color)
  (make-color (random 1.0) (random 1.0) (random 1.0)))
