;;; This file is part of guile-with-graphics, a graphic library for Guile

;;; Copyright (C) 2009  Luca Saiu

;;; This program is free software: you can redistribute it and/or modify
;;; it under the terms of the GNU General Public License as published by
;;; the Free Software Foundation, either version 3 of the License, or
;;; (at your option) any later version.

;;; This program is distributed in the hope that it will be useful,
;;; but WITHOUT ANY WARRANTY; without even the implied warranty of
;;; MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
;;; GNU General Public License for more details.

;;; You should have received a copy of the GNU General Public License
;;; along with this program.  If not, see <http://www.gnu.org/licenses/>.


(initialize-graphics "Interactive demo" 800 600)

(load "color.scm") ;; this must be loaded *after* graphics is initialized
(load "utility.scm")
(load "graphics.scm")
(load "interactive.scm")


(define block-width 80)
(define block-height 25)
(define block-color (dark red))
(define block-x (/ (window-width) 2))
(define block-y (/ (window-height) 2))
(define delta-x 0)
(define delta-y 0)
(define changing-color #f)
(define time-at-startup (time))
(define tick-length 30)
(define speed (* 0.2 tick-length))

(define (handle-event event)
  (cond ((or (equal? event '(quit))
             (equal? event '(key-down #\q))
             (equal? event '(key-down escape)))
         (exit 0))
        ((equal? event '(key-down left))
         (set! delta-x (- speed)))
        ((equal? event '(key-up left))
         (set! delta-x 0))
        ((equal? event '(key-down right))
         (set! delta-x speed))
        ((equal? event '(key-up right))
         (set! delta-x 0))
        ((equal? event '(key-down down))
         (set! delta-y speed))
        ((equal? event '(key-up down))
         (set! delta-y 0))
        ((equal? event '(key-down up))
         (set! delta-y (- speed)))
        ((equal? event '(key-up up))
         (set! delta-y 0))
        ((equal? event '(key-up space))
         (set! changing-color #f)) 
        ((equal? event '(key-down space))
         (set! changing-color #t))
        (else
         (format #t "Unhandled event: ~a\n" event))))

(define (on-tick)
  (set! block-x (+ block-x delta-x))
  (set! block-y (+ block-y delta-y))
  (if changing-color
      (set! block-color (random-color)))
  (draw-block block-x block-y block-width block-height block-color)
  (refresh)
  (draw-rectangle (- block-x 1) (- block-y 1) (+ block-width 2) (+ block-height 2)
                  black))


(display "=============================================\n")
(display "  Use the arrow keys to move the block, and\n")
(display "  the space key to change its color.\n")
(display "=============================================\n")
(clear black)
(refresh)
(interactive-game tick-length
                  handle-event
                  on-tick)
