;;; This file is part of guile-with-graphics, a graphic library for Guile

;;; Copyright (C) 2009  Luca Saiu

;;; This program is free software: you can redistribute it and/or modify
;;; it under the terms of the GNU General Public License as published by
;;; the Free Software Foundation, either version 3 of the License, or
;;; (at your option) any later version.

;;; This program is distributed in the hope that it will be useful,
;;; but WITHOUT ANY WARRANTY; without even the implied warranty of
;;; MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
;;; GNU General Public License for more details.

;;; You should have received a copy of the GNU General Public License
;;; along with this program.  If not, see <http://www.gnu.org/licenses/>.


;;(load "utility.scm")
;;(load "graphics.scm")

;;; This is useful for simple tests, not for interactive-game; notice that
;;; this procedure handles all pending events (not just one) and it has
;;; zero parameters; hence it's *not* suitable to be used as a parameter
;;; of handle-all-enqueued-events:
(define (handle-quit-event-only)
  (handle-all-enqueued-events (lambda (event)
                                (if (or (equal? event '(quit))
                                        (equal? event '(key-down #\q))
                                        (equal? event '(key-down escape)))
                                    (exit)
                                    #t))))

(define (handle-all-enqueued-events handle-one-event)
  (let ((next-event-or-#f (get-event)))
    (if next-event-or-#f
        (begin
          (handle-one-event next-event-or-#f)
          (handle-all-enqueued-events handle-one-event))
        'no-more-events)))

(define (interactive-game tick-length handle-event on-tick)
  (let* ((time-at-startup (time))
         (last-tick-time time-at-startup))
    (while #t
           (let* ((current-time (time))
                  (time-since-last-tick (- current-time last-tick-time))
                  (ticks-no-since-last-update
                   (inexact->exact (floor (/ time-since-last-tick tick-length)))))
             (if (not (zero? ticks-no-since-last-update))
                 (begin
                   (handle-all-enqueued-events handle-event)
                   (for i 1 ticks-no-since-last-update
                        (on-tick)
                   (set! last-tick-time current-time))))
             (let ((ticks-no-till-next-update (remainder (time) tick-length)))
               (usleep ticks-no-till-next-update))))))
